use actix_web::{web, HttpResponse, Responder};
use chrono::NaiveDate;
use rinja::Template;
use serde::Deserialize;
use sqlx::PgPool;

use crate::endpoints::availability::{calc_free_slots_cor, NewOrEditAvailabilityTemplate};
use crate::models::{Availability, User};
use crate::utils::ApplicationError;

#[derive(Deserialize)]
struct AvailabilityNewQuery {
    #[serde(rename(deserialize = "wholeday"))]
    whole_day: Option<bool>,
    date: NaiveDate,
}

#[actix_web::get("/availabillity/new")]
pub async fn get(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    query: web::Query<AvailabilityNewQuery>,
) -> Result<impl Responder, ApplicationError> {
    let availabillities = Availability::read_by_date_and_area_including_user(
        pool.get_ref(),
        query.date,
        user.area_id,
    )
    .await?;

    let availabilities_from_user: Vec<&Availability> = availabillities
        .iter()
        .filter(|a| a.user_id == user.id)
        .collect();
    let only_one_availability_is_wholeday = availabilities_from_user.len() == 1
        && availabilities_from_user[0].start_time.is_none()
        && availabilities_from_user[0].end_time.is_none();

    let user_can_create_availabillity = availabilities_from_user.len() == 0
        || !(only_one_availability_is_wholeday
            || calc_free_slots_cor(&availabilities_from_user).len() == 0);

    if !user_can_create_availabillity {
        return Ok(HttpResponse::BadRequest().finish());
    }

    let template = NewOrEditAvailabilityTemplate {
        user: user.into_inner(),
        date: query.date,
        whole_day: query.whole_day.unwrap_or(true),
        id: None,
        start_time: None,
        end_time: None,
        comment: None,
        slot_suggestions: calc_free_slots_cor(&availabilities_from_user),
    };

    Ok(HttpResponse::Ok().body(template.render()?))
}
