use crate::{
    endpoints::availability::calc_free_slots_cor, filters, models::{Assignment, Function, Vehicle}, utils::{event_planning_template::generate_vehicles_assigned_and_available, ApplicationError}
};
use actix_web::{web, HttpResponse, Responder};
use chrono::{NaiveDate, NaiveTime, Utc};
use rinja::Template;
use serde::Deserialize;
use sqlx::PgPool;

use crate::models::{Area, Availability, Event, Role, User};

#[derive(Deserialize)]
pub struct CalendarQuery {
    date: Option<NaiveDate>,
    area: Option<i32>,
}

#[derive(Template)]
#[template(path = "index.html")]
struct CalendarTemplate {
    user: User,
    user_can_create_availabillity: bool,
    date: NaiveDate,
    selected_area: Option<i32>,
    areas: Vec<Area>,
    events_and_assignments: Vec<(
        Event,
        Vec<String>,
        Option<String>,
        Option<String>,
        Vec<Vehicle>,
    )>,
    availabillities: Vec<Availability>,
}

#[actix_web::get("/")]
async fn get(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    query: web::Query<CalendarQuery>,
) -> Result<impl Responder, ApplicationError> {
    let date = match query.date {
        Some(given_date) => given_date,
        None => Utc::now().date_naive(),
    };

    let areas = Area::read_all(pool.get_ref()).await?;

    let selected_area = match query.area {
        Some(id) => {
            if !areas.iter().any(|a| a.id == id) {
                return Ok(HttpResponse::BadRequest().finish());
            }

            Some(id)
        }
        None => None,
    };

    let availabillities = Availability::read_by_date_and_area_including_user(
        pool.get_ref(),
        date,
        query.area.unwrap_or(user.area_id),
    )
    .await?;

    let availabilities_from_user: Vec<&Availability> = availabillities
        .iter()
        .filter(|a| a.user_id == user.id)
        .collect();
    let only_one_availability_is_wholeday = availabilities_from_user.len() == 1
        && availabilities_from_user[0].start_time.is_none()
        && availabilities_from_user[0].end_time.is_none();

    let user_can_create_availabillity = availabilities_from_user.len() == 0
        || !(only_one_availability_is_wholeday
            || calc_free_slots_cor(&availabilities_from_user).len() == 0);

    let mut events_and_assignments = Vec::new();
    for e in Event::read_all_by_date_and_area_including_location(
        pool.get_ref(),
        date,
        query.area.unwrap_or(user.area_id),
    )
    .await?
    .into_iter()
    {
        let assignments = Assignment::read_all_by_event(pool.get_ref(), e.id).await?;
        let (posten, rest): (Vec<Assignment>, Vec<Assignment>) = assignments
            .into_iter()
            .partition(|a| a.function == Function::Posten);
        let (wachhabender, fuehrungsassistent): (Vec<Assignment>, Vec<Assignment>) = rest
            .into_iter()
            .partition(|a| a.function == Function::Wachhabender);

        let (assigned_vehicle, _) = generate_vehicles_assigned_and_available(&pool, &e).await?;

        events_and_assignments.push((
            e,
            posten
                .into_iter()
                .map(|p| {
                    availabillities
                        .iter()
                        .find(|a| a.id == p.availabillity_id)
                        .unwrap()
                        .user
                        .as_ref()
                        .unwrap()
                        .name
                        .clone()
                })
                .collect(),
            fuehrungsassistent.first().and_then(|fa| {
                Some(
                    availabillities
                        .iter()
                        .find(|a| a.id == fa.availabillity_id)
                        .unwrap()
                        .user
                        .as_ref()
                        .unwrap()
                        .name
                        .clone(),
                )
            }),
            wachhabender.first().and_then(|wh| {
                Some(
                    availabillities
                        .iter()
                        .find(|a| a.id == wh.availabillity_id)
                        .unwrap()
                        .user
                        .as_ref()
                        .unwrap()
                        .name
                        .clone(),
                )
            }),
            assigned_vehicle,
        ));
    }

    let template = CalendarTemplate {
        user: user.into_inner(),
        user_can_create_availabillity,
        date,
        selected_area,
        areas,
        events_and_assignments,
        availabillities,
    };

    Ok(HttpResponse::Ok().body(template.render()?))
}
