use actix_web::{web, HttpResponse, Responder};
use rinja::Template;
use serde::Deserialize;
use sqlx::PgPool;

use crate::{
    endpoints::{
        availability::{calc_free_slots_cor, NewOrEditAvailabilityTemplate},
        IdPath,
    },
    models::{Availability, User},
    utils::ApplicationError,
};

#[derive(Deserialize)]
struct EditAvailabilityQuery {
    #[serde(rename(deserialize = "wholeday"))]
    whole_day: Option<bool>,
}

#[actix_web::get("/availabillity/edit/{id}")]
pub async fn get(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
    query: web::Query<EditAvailabilityQuery>,
) -> Result<impl Responder, ApplicationError> {
    let Some(availabillity) = Availability::read_by_id(pool.get_ref(), path.id).await? else {
        return Ok(HttpResponse::NotFound().finish());
    };

    if availabillity.user_id != user.id {
        return Err(ApplicationError::Unauthorized);
    }

    let start_time = availabillity
        .start_time
        .and_then(|d| Some(d.format("%R").to_string()));

    let end_time = availabillity
        .end_time
        .and_then(|d| Some(d.format("%R").to_string()));

    let has_time = availabillity.start_time.is_some() && availabillity.end_time.is_some();

    let suggestions = if has_time {
        let availabillities = Availability::read_by_date_and_area_including_user(
            pool.get_ref(),
            availabillity.date,
            user.area_id,
        )
        .await?;

        let availabilities_from_user: Vec<&Availability> = availabillities
            .iter()
            .filter(|a| a.user_id == user.id)
            .collect();

        calc_free_slots_cor(&availabilities_from_user)
            .into_iter()
            .filter(|(a, b)| {
                *b == availabillity.start_time.unwrap() || *a == availabillity.end_time.unwrap()
            })
            .collect()
    } else {
        Vec::new()
    };

    let template = NewOrEditAvailabilityTemplate {
        user: user.into_inner(),
        date: availabillity.date,
        whole_day: query.whole_day.unwrap_or(!has_time),
        id: Some(path.id),
        start_time: start_time.as_deref(),
        end_time: end_time.as_deref(),
        comment: availabillity.comment.as_deref(),
        slot_suggestions: suggestions,
    };

    Ok(HttpResponse::Ok().body(template.render()?))
}
